Attribute VB_Name = "Module1"


Rem ----------------------------------------------------------------------------
Rem -- ABOUT THIS MODULE
Rem -- -----------------
Rem --
Rem -- This module contains global constants, public variables, and public
Rem -- function used throughout this sample application.
Rem --
Rem ----------------------------------------------------------------------------

Rem ----------------------------------------------------------------------------
Rem -- DISCLAIMER AND DISTRIBUTION
Rem -- ---------------------------
Rem --
Rem -- All code, programming styles, and architecture are provided for
Rem -- demonstration purposes only, and may not be suitable for every project.
Rem --
Rem -- The code contained in this sample application may be used freely within
Rem -- third-party applications if the following criteria are met:
Rem --
Rem -- 1. This sample code must be a part of a larger application.
Rem -- 2. This sample code may not be packaged and distributed as a commercial
Rem --    product.
Rem ----------------------------------------------------------------------------

Rem ----------------------------------------------------------------------------
Rem -- Global Constants
Rem ----------------------------------------------------------------------------


Rem ----------------------------------------------------------------------------
Rem -- Private Constants
Rem ----------------------------------------------------------------------------

' The following constants are used to find the eSignal Data Manager
' application window, and to close the application.
'
'   NOTE:  This sample application only attempts to close the eSignal Data
'          Manager when it has launched the eSignal Data Manager.  If the
'          eSignal Data Manager was running prior to execution of this sample
'          application, then the eSignal Data Manager is left running upon
'          exit of this sample application.
Private Const GW_HWNDNEXT = 2
Private Const GW_CHILD = 5
Private Const GWL_HWNDPARENT = (-8)
Private Const WM_CLOSE = &H10
Private Const applicationName = "eSignal Data Manager"

Rem ----------------------------------------------------------------------------
Rem -- Public Variables
Rem ----------------------------------------------------------------------------

Public g_username
Public g_password
Public g_internetAddress

Rem ----------------------------------------------------------------------------
Rem -- Private Variables
Rem ----------------------------------------------------------------------------

' The following private variables are used for parsing the WINROS.INI file.
Dim windir$
Dim dmApp$
Dim dmIni$
Dim task_id As Variant

Rem ----------------------------------------------------------------------------
Rem -- TYPE DEFINITIONS
Rem ----------------------------------------------------------------------------
Type FILETIME
   dwLowDateTime As Long
   dwHighDateTime As Long
End Type

Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
End Type

Rem ----------------------------------------------------------------------------
Rem ----------------------------------------------------------------------------
Rem -- FUNCTION DEFINITIONS
Rem ----------------------------------------------------------------------------
Rem ----------------------------------------------------------------------------

' The following functions are used to get information from the WINROS.INI file.
' These functions are not part of the eSignal Control or the standard
' eSignal API.  These functions are part of the Windows API.

Declare Function GetPrivateProfileString Lib "kernel32" _
   Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, _
                                     ByVal lpKeyName As Any, _
                                     ByVal lpDefault As String, _
                                     ByVal lpReturnedString As String, _
                                     ByVal nSize As Long, _
                                     ByVal lpFileName As String) As Long
                                                                            
Declare Function GetWindowsDirectory Lib "kernel32" _
   Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, _
                                 ByVal nSize As Long) As Long
                                 
Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

' The following functions are used to launch and terminate the
' eSignal Data Manager application.
' These functions are not part of the eSignal Control or the standard
' eSignal API.  These functions are part of the Windows API.

Declare Function GetDesktopWindow Lib "user32" () As Long

Declare Function GetWindow Lib "user32" (ByVal hWnd As Long, _
                                         ByVal wCmd As Long) As Long
                                         
Declare Function GetWindowText Lib "user32" _
   Alias "GetWindowTextA" (ByVal hWnd As Long, _
                           ByVal lpString As String, _
                           ByVal cch As Long) As Long

Declare Function GetWindowThreadProcessId Lib "user32" (ByVal hWnd As Long, _
                                                        procId As Long) As Long
   
Declare Function GetParent Lib "user32" (ByVal hWnd As Long) As Long

Declare Function FindWindow Lib "user32" _
   Alias "FindWindowA" (ByVal lpClassName As String, _
                        ByVal lpWindowName As String) As Long

Declare Function SendMessage Lib "user32" _
   Alias "SendMessageA" (ByVal hWnd As Long, _
                         ByVal wMsg As Long, _
                         ByVal wParam As Long, _
                         ByVal lParam As Any) As Long
                         
Declare Function GetWindowLong Lib "user32" _
   Alias "GetWindowLongA" (ByVal hWnd As Long, _
                           ByVal nIndex As Long) As Long
                           
' The following function is used to convert the UTC time to local time.
Declare Function FileTimeToLocalFileTime Lib "kernel32.dll" (lpFileTime As FILETIME, _
                                                             lpLocalFileTime As FILETIME) As Long
                                                             
Declare Function SystemTimeToFileTime Lib "kernel32.dll" (lpSystemTime As SYSTEMTIME, _
                                                          lpFileTime As FILETIME) As Long
                                                          
Declare Function FileTimeToSystemTime Lib "kernel32.dll" (lpFileTime As FILETIME, _
                                                          lpSystemTime As SYSTEMTIME) As Long
                                                          
                                                          
Rem ----------------------------------------------------------------------------
Rem ----------------------------------------------------------------------------
Rem -- FUNCTION IMPLEMENTATIONS
Rem ----------------------------------------------------------------------------
Rem ----------------------------------------------------------------------------

Rem --------------------------------------------------------------------------
Rem Function: Starts the local eSignal Data Manager if it is not already
Rem           running.
Rem
Rem    Usage: Used at the start of the application to ensure connectivity
Rem           to the data servers.
Rem
Rem  Returns: TRUE if the local eSignal Data Manager is already running, or
Rem           if the sample application successfully starts it; FALSE
Rem           otherwise.
Rem
Rem     Side
Rem  Effects: The following private variable is set by this function:
Rem              task_id
Rem
Rem           If the local eSignal Data Manager was already running before
Rem           the sample application executed, then task_id is set to -1.
Rem --------------------------------------------------------------------------
Function startDataManager() As Boolean
   On Error GoTo ShellError
   
   If Not dm_IsRunning(0, applicationName) Then
      If ParseINI() Then
         task_id = Shell(dmApp$, vbMinimizedNoFocus)
         If dm_IsRunning(0, applicationName) Then
            startDataManager = True
         Else
            startDataManager = False
         End If
      End If
   Else
      task_id = -1
      startDataManager = True
   End If
   Exit Function
   
ShellError:
   MsgBox "Could not start " + target_path & vbCrLf & Err.Description, vbOKOnly Or vbExclamation, "Error"
   startDataManager = False
   Exit Function
End Function


Rem --------------------------------------------------------------------------
Rem Function: Stops the eSignal Data Manager
Rem
Rem    Usage: Used upon exit of the sample application.
Rem
Rem  Returns: TRUE if the local eSignal Data Manager was stopped by the
Rem           sample application; FALSE otherwise.  If the task_id is -1, then
Rem           this function returns TRUE, because the sample application is
Rem           not expected to stop the eSignal Data Manager if it were
Rem           running prior to execution of the sample application.
Rem --------------------------------------------------------------------------
Function stopDataManager() As Boolean
   Dim hWnd As Long
   
   If task_id <> -1 Then
      hWnd = InstanceToWnd(task_id)
      
      If GetWindowLong(hWnd, GWL_HWNDPARENT) = 0 Then
         Call SendMessage(hWnd, WM_CLOSE, 0&, ByVal 0&)
         stopDataManager = True
      Else
         stopDataManager = False
      End If
   Else
      stopDataManager = True
   End If
End Function


Rem --------------------------------------------------------------------------
Rem Function: This function parses the WINROS.INI file for the
Rem           WINROS.EXE startpath, and for the username and password for the
Rem           Data Manager account.
Rem
Rem    Usage: Used from within this module by other public functions.
Rem
Rem  Returns: TRUE if the WINROS.INI file was parsed successfully; FALSE
Rem           otherwise.
Rem
Rem     Side
Rem  Effects: This function sets the values for the following private and
Rem           public global variables:
Rem
Rem           windir$, dmApp$, dmIni$, g_username, g_password, g_internetAddress
Rem --------------------------------------------------------------------------
Function ParseINI() As Boolean
   Dim returnstring$
   Dim size%
      
   On Error GoTo Error
   ' Get the path to the main windows system directory where most INI files
   ' are stored.
   returnstring$ = Space(128)
   size% = GetWindowsDirectory(returnstring$, 128)
   windir$ = Left$(returnstring$, size%)
   
   ' Set the path to the WINROS.INI file.
   dmIni$ = windir$ + "\winros.ini"

   ' Parse the WINROS.INI file for the username.
   returnstring$ = Space(128)
   size% = GetPrivateProfileString("winros", "InternetUserName", "", _
                                   returnstring$, 128, dmIni$)
   g_username = Left$(returnstring$, size%)

   ' Parse the WINROS.INI file for the password.
   returnstring$ = Space(128)
   size% = GetPrivateProfileString("winros", "InternetPassword", "", _
                                   returnstring$, 128, dmIni$)
   g_password = Left$(returnstring$, size%)

   ' Parse the WINROS.INI file for the password.
   returnstring$ = Space(128)
   size% = GetPrivateProfileString("winros", "InternetAddress", "", _
                                   returnstring$, 128, dmIni$)
   g_internetAddress = Left$(returnstring$, size%)
   
   ' Parse the WINROS.INI file for the start path of the eSignal Data
   ' Manager applications
   returnstring$ = Space(128)
   size% = GetPrivateProfileString("winros", "StartPath", "", _
                                   returnstring$, 128, dmIni$)
   dmApp$ = Left$(returnstring$, size%) + "\winros.exe"

   ' The WINROS.INI file has been parsed for the required data.
   ParseINI = True
   Exit Function
Error:
   ParseINI = False
End Function


Rem --------------------------------------------------------------------------
Rem Function: This function determines if there is an application running with
Rem           a specified window title.
Rem
Rem    Usage: Use this function to determine if the eSignal Data Manager
Rem           application is running.
Rem
Rem  Returns: This function returns TRUE if the window title is found, and
Rem           FALSE if it is not found.
Rem --------------------------------------------------------------------------
Private Function dm_IsRunning(ByVal hWndStart As Long, _
                              WindowTitle As String) As Boolean
   Dim hWnd As Long
   Dim sWindowTitle As String
   Dim Temp As Long
   
   'Used for recursion through the children associated with GetDesktopWindow().
   Static level As Integer
   
   'Gets all of the active desktop windows the first time this function
   'is called.
   If level = 0 Then
      If hWndStart = 0 Then
         hWndStart = GetDesktopWindow()
      End If
      
      'Increment the recursion counter and get the first child from the list of
      'active windows returned by the GetDesktopWindow function.
      level = level + 1
      hWnd = GetWindow(hWndStart, GW_CHILD)
      
      'Search through all of the children until a match occurs.
      Do Until hWnd = 0
         'Recursive searching
         Call dm_IsRunning(hWnd, WindowTitle)
         sWindowTitle = Space$(255)
         Temp = GetWindowText(hWnd, sWindowTitle, 255)
         sWindowTitle = Left(sWindowTitle, Temp)
         
         'There was a match.  Return TRUE to call originator and exit this loop.
         If (sWindowTitle Like WindowTitle) Then
            dm_IsRunning = True
            
            'Exits the loop.
            hWnd = 0
         End If
         
         'Get the next child in the list.
         hWnd = GetWindow(hWnd, GW_HWNDNEXT)
      Loop
      
      level = level - 1
   End If
End Function


Rem --------------------------------------------------------------------------
Rem Function: Gets the window handle for the local eSignal Data Manager.
Rem
Rem    Usage: Used when the sample application attempts to shut down the
Rem           local eSignal Data Manager.
Rem
Rem  Returns: The handle to the local eSignal Data Manager application
Rem           window.
Rem --------------------------------------------------------------------------
Private Function InstanceToWnd(ByVal target_pid As Long) As Long
   Dim test_hwnd As Long
   Dim test_pid As Long
   Dim test_thread_id As Long
   
   test_hwnd = FindWindow(vbNullString, "eSignal Data Manager")
   
   Do While test_hwnd <> 0
      If GetParent(test_hwnd) = 0 Then
         test_thread_id = GetWindowThreadProcessId(test_hwnd, test_pid)
         If test_pid = target_pid Then
            InstanceToWnd = test_hwnd
            Exit Do
         End If
      End If
      
      test_hwnd = GetWindow(test_hwnd, GW_HWNDNEXT)
   Loop
End Function

Rem --------------------------------------------------------------------------
Rem Function: Converts a date into a SYSTEMTIME data structure.
Rem
Rem    Usage: Used by the Ticker objects to display the time properly.
Rem
Rem  Returns: A SYSTEMTIME structure by reference.
Rem
Rem     Note: Code from http://www.vb-helper.com/howto_utc_to_local_time.html
Rem --------------------------------------------------------------------------
Private Sub DateToSystemTime(ByVal the_date As Date, ByRef system_time As SYSTEMTIME)
    With system_time
        .wYear = Year(the_date)
        .wMonth = Month(the_date)
        .wDay = Day(the_date)
        .wHour = Hour(the_date)
        .wMinute = Minute(the_date)
        .wSecond = Second(the_date)
    End With
End Sub

Rem --------------------------------------------------------------------------
Rem Function: Converts a SYSTEMTIME into a date.
Rem
Rem    Usage: Used by the Ticker objects to display the time properly.
Rem
Rem  Returns: A Date structure by reference.
Rem
Rem     Note: Code from http://www.vb-helper.com/howto_utc_to_local_time.html
Rem --------------------------------------------------------------------------
Private Sub SystemTimeToDate(system_time As SYSTEMTIME, ByRef the_date As Date)
    With system_time
        the_date = CDate( _
            Format$(.wMonth) & "/" & _
            Format$(.wDay) & "/" & _
            Format$(.wYear) & " " & _
            Format$(.wHour) & ":" & _
            Format$(.wMinute, "00") & ":" & _
            Format$(.wSecond, "00"))
    End With
End Sub

Rem --------------------------------------------------------------------------
Rem Function: Converts a UTC time to local time.
Rem
Rem    Usage: Used by the Ticker objects to display the time properly.
Rem
Rem  Returns: The local time for a given UTC time.
Rem
Rem     Note: Code from http://www.vb-helper.com/howto_utc_to_local_time.html
Rem --------------------------------------------------------------------------
Function UTCToLocalTime(ByVal the_date As Date) As Date
   Dim system_time As SYSTEMTIME
   Dim local_file_time As FILETIME
   Dim utc_file_time As FILETIME

   ' Convert it into a SYSTEMTIME.
   DateToSystemTime the_date, system_time

   ' Convert it to a UTC time.
   SystemTimeToFileTime system_time, utc_file_time
    
   ' Convert it to a FILETIME.
   FileTimeToLocalFileTime utc_file_time, local_file_time

   ' Convert it to a SYSTEMTIME.
   FileTimeToSystemTime local_file_time, system_time

   ' Convert it to a Date.
   SystemTimeToDate system_time, the_date

   UTCToLocalTime = the_date
End Function

'Private Const APP_NAME As String = "howto_persist_position"
'Public Sub getPositon(whichForm)
' Restore the form's size and position.
 '   whichForm.Move _
 '       GetSetting(APP_NAME, "Settings", "Left", whichForm.Left), _
 '       GetSetting(APP_NAME, "Settings", "Top", whichForm.Top), _
 '       GetSetting(APP_NAME, "Settings", "Width", _
 '           whichForm.Width), _
  '      GetSetting(APP_NAME, "Settings", "Height", _
   '         whichForm.Height)
'End Sub

'Public Sub savePosition(whichForm)
 ' Save the form's size and position.
   ' SaveSetting APP_NAME, "Settings", "Left", whichForm.Left
  '  SaveSetting APP_NAME, "Settings", "Top", whichForm.Top
   ' SaveSetting APP_NAME, "Settings", "Width", whichForm.Width
   ' SaveSetting APP_NAME, "Settings", "Height", whichForm.Height
'End Sub



